var cmd = require('./commands.js');
var con = require('./database.js').connection;
var panel = require('./panel.js');

var latestMaps = [];
var latestMapsLimit = 7;
var jukebox = [];

exports.Init = function(core) {
	// Check if admin plugin is loaded.
	if (!core.isPluginLoaded('commands.js')) {
		console.log(' !> Plugin [commands.js] is not loaded, which is required.');
		return false;
	}
	// Check if admin db is loaded.
	if (!core.isPluginLoaded('database.js')) {
		console.log(' !> Plugin [database.js] is not loaded, which is required.');
		return false;
	}
	
	core.onStatusChanged(statusChanged);
	core.callMethod('GetCurrentMapInfo', [], function(core, params) {
		mapChanged(core, params[0]);
	});
	core.onRulesScriptCallback(function(core, params) {
		//console.log(params[0], params[1]);
		if (params[0] == "MapLoaded") {
			core.callMethod('GetCurrentMapInfo', [], function(core, params) {
				mapChanged(core, params[0]);
			});
		}
	});
	
	cmd.onCommand('list', listMaps, 0, "List all maps on the server");
	cmd.onCommand('jukebox', listJukebox, 0, "List the jukeboxed maps");
	cmd.onCommand('me', function(core, login, params_, complete) {
		core.callMethod('GetDetailedPlayerInfo', [login], function(core, params) {
			core.callMethod('ChatSendServerMessage', ['$z'+params[0]['NickName']+"$z$i$f80"+complete])
		});
	}, 0, "Emote yourself");
	
	core.on('MapLoaded', function(core, params) { // params = array of mapdata
		if (latestMaps.length == latestMapsLimit)
			latestMaps.shift();
		latestMaps.push(params['FileName']);
	});
	core.on('MatchEnded', function(core, params) { // params = []
		if (jukebox.length > 0) { // select map from jukebox
			var data = jukebox[0];
			jukebox.shift();
			core.callMethod('ChooseNextMap', [data[1]]);
			core.callMethod('ChatSendServerMessage', ['$z$o$fff» $o$i$s$08fThe next map will be '+data[2]+'$z$i$s$08f as requested by '+data[3]]);
		}
	});
	core.onPlayerDisconnect(function(core, params) {
		for (i in jukebox)
			if (jukebox[i][0] == params[0]) {
				jukebox.splice(i,1);
				break;
			}
	});

	return true;
}

function statusChanged(core, params) {
	if (params[0] != 4) // Playing
		return;
	core.callMethod('GetCurrentMapInfo', [], function(core, params) {
		mapChanged(core, params[0]);
	});
}

function mapChanged(core, mapdata) {
	con.query('SELECT * FROM maps WHERE uid=\''+mapdata['UId']+'\'', function(err, rows, fields) {
		if (err) throw err;
		if (rows.length == 0)
			con.query('INSERT INTO maps SET ?', {name: mapdata['Name'], uid: mapdata['UId'], author: mapdata['Author']});
	});
}

function listMaps(core, login, param) {
	core.callMethod('GetMapList', [-1, 0], function(core, params) {
		var tracklist = [];
		for (i in params[0]) {
			tracklist.push([(tracklist.length+1)+'.', params[0][i]['Name'], params[0][i]['Author'], '[Add]', params[0][i]['FileName']]);
		}
		panel.ShowList(core, login, {subject: 'Map list', columns_widths: [0.05, 0.55, 0.25, 0.15, 0], 
			columns_names: ['#', 'Track name', 'Author', 'Jukebox'], callbacks: [null, null, null, jukebox_add]}, tracklist, 20);
	});
}

function listJukebox(core, login, param) {
	var tracklist = [];
	for (i in jukebox) {
		tracklist.push([(tracklist.length+1)+'.', jukebox[i][2], jukebox[i][3], '[Remove]', jukebox[i][1]]);
	}
	panel.ShowList(core, login, {subject: 'Jukebox list', columns_widths: [0.05, 0.45, 0.30, 0.20, 0], 
		columns_names: ['#', 'Track name', 'Requested by', 'Remove'], callbacks: [null, null, null, jukebox_remove]}, tracklist, 20);
}

function jukebox_add(core, login, row) {
	if (latestMaps.indexOf(row[4]) != -1) {
		core.callMethod('ChatSendServerMessageToLogin', ['$z$o$fff» $o$i$s$4f4Can not add '+row[1]+'$z$i$s$4f4 to the jukebox because it\'s recently played.', login]);
	} else {
		for (i in jukebox)
			if (jukebox[i][1] == row[4]) {
				core.callMethod('ChatSendServerMessageToLogin', ['$z$o$fff» $o$i$s$4f4Map '+row[1]+'$z$i$s$4f4 is already in the jukebox.', login]);
				return;
			} else if (jukebox[i][0] == login) {
				core.callMethod('ChatSendServerMessageToLogin', ['$z$o$fff» $o$i$s$4f4You already have a map in the jukebox.', login]);
				return;
			}
		core.callMethod('GetDetailedPlayerInfo', [login], function(core, params) {
			core.callMethod('ChatSendServerMessageToLogin', ['$z$o$fff» $o$i$s$4f4You have added map '+row[1]+'$z$i$s$4f4 to the jukebox.', login]);
			core.callMethod('ChatSendServerMessage', ['$z$o$fff» '+params[0]['NickName']+'$z$i$s$08f added the map '+row[1]+'$z$i$s$08f to the jukebox.']);
			jukebox.push([login, row[4], row[1], params[0]['NickName']]);
			listJukebox(core, login);
		});
	}
}

function jukebox_remove(core, login, row) {
	for (i in jukebox)
		if (jukebox[i][0] == login && jukebox[i][1] == row[4]) {
			core.callMethod('ChatSendServerMessageToLogin', ['$z$o$fff» $o$i$s$4f4You have removed map '+row[1]+'$z$i$s$4f4 from the jukebox.', login]);
			jukebox.splice(i,1);
			listJukebox(core, login);
			return;
		}
	core.callMethod('ChatSendServerMessageToLogin', ['$z$o$fff» $o$i$s$4f4Can not remove map '+row[1]+'$z$i$s$4f4 from the jukebox.', login]);
}